# This is a data storage file for JUnitStats Mod by Jeckel.
# The variable 'TypesDict' and method 'getRoll' are both
# called outside this file.
# All other variables are for internal use only.
#
# Use this file as an example to create Custom Stats Files.

from CvPythonExtensions import *
import CvUtil
import sys

import JUtils

############################################################################
# globals

gc = CyGlobalContext()

#######################################################################################################

# How a Types Dictionary is broken down.
# This is the same for both the Base Types Dictionary and Custom Types Dictionaries.
# First, obviously, it is a python dictionary. In other languages this is referded to
# as a hash, map, or multi-deliminated list and will here after be refered to as dict
# or dicts.

# The keys of this dict are strings that represent a category of stats and are termed
# Types. An example from World or Darkness, "Physical Attributes" and "Talent" would
# both be exceptable, but "Wits" or "Melee" would not. All Types must be unique and
# strings are preferred, though int might go in fine but could be changed to strings at
# some point.

# Each Type has as its Value a list.
# This list is termed the StatsList and the list items themselves are termed Stats.
# The Stats in the StatsList follow the same rules as Types, though Stats are only
# required to be unique inside their Type.
# For example, you could have the Stat 'Talent' in the 'Hand2Hand' Type and another Stat
# 'Talent'in 'Driving' Type. But you can Not have 2 Stats called 'Talent' in 'Hand2Hand'
# Type or 2 Types called 'Driving'.

# A very simple example of a TypesDict:
# TypesDict = { "Type1" : [ "Stat1", "Stat2" ], "Type2" : [ "Stat1", "Stat2" ] }

# Using the World of Darkness Attributes as an example:
# TypesDict = { "Physical Attributes" : [ "Strength", "Dexterity", "Stamina" ],
#               "Social Attributes" : [ "Appearance", "Manipulation", "Charisma" ],
#               "Mental Attributes" : [ "Perception", "Intelligence", "Wits" ]
#               }

# On the other hand, an AD&D TypesDict could be as simple as this:
# TypesDict = { "UnitAttributes" : [ "STR", "DEX", "CON", "INT", "WIS", "CHA" ] }

# Setting up the TypesDict is the easy part and is just the base on which we build
# a unit's acual dictionary of Types and their Stat/Value pairs.

# The important part is how we determine a Stat's inital value when a unit is created.

# Most basicly, if all else fails, a Stat will be set to 0.
# The main way a Stat's initial Value is determined is by the calling of that StatsData
# file's default() method.
# In addition to using the default() method, as each stat is set on a unit the code looks
# in the StatsData file for that Stat and checks if there is a funtion named the same as
# the Stat, it is case sensative. If one is found it will use that method's result as the
# Value of the Stat.

# Useing the AD&D example. my default() method could simply return a random number between
# 3 and 19 and that gets used for most of the Stats. But I could then also have a method
# defined as Strength(), making sure to spell and capitalize it just as the Stat is named.
# In that case, the Strength() method would be called for the Strength Stat, but the rest
# of the Stats would call default() method.

# This StatsData file contains 1 Type and 7 Stats.
# 5 of the Stats use custom roll methods, the other two use defaultRoll().
# Physical Stats:
#   Strength: How strong you are.
#   Dexterity: How well you can control your movements.
#   Constitution: How well you deal with damage.
#   Stamina: How "inshape" you are and how long you can sustain physically exertion.
#   Perception: How well you can see.
#   Height: How tall you are.
#   Weight: How much you weigh.

#######################################################################################################

# Physical Stats
#PHYSICAL_STATS = 
#STRENGTH = 
#DEXTERITY = 
#CONSTITUTION = 
#STAMINA = 
#PERCEPTION = 
#HEIGHT = 
#WEIGHT = 


# strStatsType : {statName : [argsList]}
TypesDict = { "PhysicalStats" : ["strength", "dexterity",
                                 "constitution", "stamina",
                                 "perception",
                                 "height", "weight"] }

def getRoll(strStat, pUnit):

        try:
                strMethod = str(strStat) + str("Roll")
                evalMethod = eval(strMethod)
                iRoll = evalMethod(pUnit)
                return iRoll
        except NameError:
                try:
                        return defaultRoll(pUnit)
                except:
                        return 0


def defaultRoll(pUnit):
        #JUtils.alert(0, "defaultRoll Ran")
        return 1

############################################################################################################

def strengthRoll(pUnit):
        #JUtils.alert(0, "strengthRoll Ran")
        return pUnit.baseCombatStr()

def dexterityRoll(pUnit):
        #JUtils.alert(0, "dexterityRoll Ran")
        return pUnit.baseMoves() + pUnit.withdrawalProbability()

def perceptionRoll(pUnit):
        #JUtils.alert(0, "perceptionRoll Ran")
        return pUnit.visibilityRange()

def heightRoll(pUnit):
        #JUtils.alert(0, "heightRoll Ran")
        iRandFeet = CyGame().getSorenRandNum(8, "Random Roll Unit Height Feet")
        iRandInches = CyGame().getSorenRandNum(100, "Random Roll Unit Height Inches")
        if (iRandFeet < 3):
                return (iRandFeet + 3.1) + (iRandInches / 100.0)
                
        return iRandFeet + (iRandInches / 100.0)

def weightRoll(pUnit):
        #JUtils.alert(0, "weightRoll Ran")
        iRandPounds1 = CyGame().getSorenRandNum(401, "Random Roll Unit Weight Pounds 1")
        iRandPounds2 = CyGame().getSorenRandNum(53, "Random Roll Unit Weight Pounds 2")
        iRand = CyGame().getSorenRandNum(1000, "Random Roll Unit Weight")
        if (iRandPounds1 < 15):
                iPounds = 15 + iRandPounds2
        else:
                iPounds = iRandPounds1 + iRandPounds2

        return iPounds + (iRand / 1000.0)













